#ifndef _GRAPH_UTILS_H
#define _GRAPH_UTILS_H

#include <list>
#include <vector>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>

using namespace std;

using boost::vertices;
using boost::edges;
using boost::get;
using boost::vertex_index;
using boost::add_edge;
using boost::remove_edge;

struct vertinfo {
  int annotation;
};

/// graph type
typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::bidirectionalS> dgraph_t;
typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS, vertinfo> ugraph_t;

typedef boost::graph_traits<ugraph_t>::vertex_iterator uv_iter;
typedef boost::graph_traits<ugraph_t>::vertex_descriptor uv_t;
typedef boost::graph_traits<ugraph_t>::edge_descriptor ue_t;
typedef boost::graph_traits<ugraph_t>::out_edge_iterator e_iter;

typedef boost::graph_traits<dgraph_t>::vertex_iterator dv_iter;
typedef boost::graph_traits<dgraph_t>::vertex_descriptor dv_t;
typedef boost::graph_traits<dgraph_t>::edge_descriptor de_t;
typedef boost::graph_traits<dgraph_t>::out_edge_iterator out_iter;
typedef boost::graph_traits<dgraph_t>::in_edge_iterator in_iter;

struct instance;

struct con_comp {
  size_t c;
  vector<int> to_idx;
};

struct instance {
  dgraph_t d;
  ugraph_t g;
  vector<string> to_name;
  vector<con_comp> comps;
};

extern size_t (*citefunc)(const list<dv_t> &verts, const instance &inst);

size_t ucites(const list<dv_t> &verts, const instance &inst);
size_t scites(const list<dv_t> &verts, const instance &inst);
size_t cites(const instance &inst, const dv_t& v);
size_t hindex(const instance &inst, unsigned purged);
instance read_instance(const char *gname, const char *dname);
bool is_clique(const list<uv_t> &verts, instance &inst);
bool can_reach_hindex(instance &inst, unsigned h, unsigned purged);
bool can_reach_hindex_k(instance &inst, unsigned h, unsigned k, unsigned purged);
size_t hindex_ubo(const instance &inst, unsigned purged);

#endif
