#include "graph.hpp"
#include <cassert>
#include <getopt.h>
#include <cstdlib>
#include <iostream>
#include <functional>
#include <limits>
#include <iomanip>
#include <vector>

using namespace std;

int main (int argc, char *argv[]) {
  char *gname = 0;
  char *dname = 0;
  int inc;
  int k = -1;
  unsigned purged = 0;

  citefunc = &ucites;
  
  while((inc=getopt(argc, argv, "g:d:k:p:s")) != -1)
    switch (inc) {
    case 'g': gname = optarg; break;
    case 'd': dname = optarg; break;
    case 'k': k = atoi(optarg); assert(k > 0); break;
    case 'p': purged = atoi(optarg); assert(purged >= 0); break;
    case 's': citefunc = &scites;
    }
  
  if(!dname || (!dname && (k <= 0))) {
    cout << "Usage: " << argv[0] << " -d citation_graph [-g compatibility_graph] [-k number_of_allowed_merges]" << endl;
    cout << "Either a citation graph or a number of allowed merges have to be given" << endl;
    exit(EXIT_FAILURE);
  }

  if(k > 0 && citefunc == &ucites) {
    cerr << "WARNING: using unionCites measure with k." << endl
	 << "  This will not yield correct results in general." << endl
         << "  Add the -s switch to use sumCites measure instead." << endl;
  }
  
  
  instance inst(read_instance(gname, dname));
  
  size_t hmin = hindex(inst, purged);
  size_t hmax = hindex_ubo(inst, purged);
  cout << "H-Index     " << hmin << endl;
  cout << "Upper bound " << hmax << endl;

  while(hmax > hmin) {
    size_t h = ceil((hmax + hmin) / 2.0);
    cout << endl << "----- trying H-index " << h << " --------" << endl;
    bool res = 0;
    if(k > 0) res = can_reach_hindex_k(inst, h, k, purged);
    else res = can_reach_hindex(inst, h, purged);

    if(res) hmin = h;
    else hmax = h - 1;
    cout << "   hmax = " << hmax << ", hmin = " << hmin << endl;
  }
  cout << endl << "------------------------ "<<endl;
  cout << "New H-Index " << hmin << endl;

  exit(EXIT_SUCCESS);
}

