#include "graph.hpp"
#include "tables.hpp"
#include "EfficientSet.hpp"
#include "SubsetIterator.hpp"
#include <climits>

typedef int comp_vert;
typedef EfficientSet<comp_vert, uint64_t> comp_subset;

list<dv_t> to_verts(const instance &inst, comp_subset c, unsigned int component) {
  assert(component < inst.comps.size());
  list<dv_t> res;
  
  int s = c.size();
  int count = 0;
  comp_vert j = 0;
  while(count < s) {
    if(member(j, c)) {
      res.push_back(inst.comps[component].to_idx[j]);
      ++count;
    }
    ++j;
  }

  return res;
}

table d_table(instance &inst, unsigned int C, unsigned int h) {
  assert(C < inst.comps.size());

  comp_subset VC((uint64_t(1) << inst.comps[C].c) - 1);
  table D(uint64_t(1) << inst.comps[C].c);

  D[0] = 0;

  SubsetIterator<comp_subset> V_(VC), end;

  for(++V_; V_ != end; ++V_) {
    list<uv_t> verts = to_verts(inst, *V_, C);
    if(is_clique(verts, inst))
      D[(*V_).rep] = (citefunc(verts, inst) >= h);
    else D[(*V_).rep] = 0;
  }
  
  return D;
}

size_t above_h_articles(instance &inst, unsigned int C, unsigned int h) {
  assert(C < inst.comps.size());

  table D(d_table(inst, C, h));
  table T(uint64_t(1) << inst.comps[C].c);

  T[0] = 0; // 0 is the empty set

  comp_subset VC((uint64_t(1) << inst.comps[C].c) - 1);

  SubsetIterator<comp_subset> V_(VC), end;

  for(++V_; V_ != end; ++V_) {
    size_t m = 0;

    SubsetIterator<comp_subset> V__(*V_);

    for(; V__ != end; ++V__) {
      size_t cur = T[(*V__).rep] + D[(*V_ - *V__).rep];
      if(cur > m) m = cur;
    }

    T[(*V_).rep] = m;
  }
  
  return T[VC.rep];
}

struct mtable {
  size_t sets, kmax;
  vector<unsigned> tab;

  mtable(size_t _sets, size_t _kmax) : sets(_sets), kmax(_kmax) {
    tab.resize(sets * (kmax + 1));
  }

  inline unsigned get(size_t set, size_t k) {
    return tab[set * kmax + k]; 
  }

  inline unsigned store(size_t set, size_t k, unsigned val) {
    return tab[set * kmax + k] = val; 
  }
};

table d_table_k(instance &inst, unsigned h, unsigned k) {
  comp_subset VC((uint64_t(1) << inst.comps[0].c) - 1);
  table D(uint64_t(1) << inst.comps[0].c);

  D[0] = 0;

  SubsetIterator<comp_subset> V_(VC), end;

  for(++V_; V_ != end; ++V_) {
    if((*V_).size() <= k + 1) {
      list<uv_t> verts = to_verts(inst, *V_, 0);
      D[(*V_).rep] = (citefunc(verts, inst) >= h);
    } else D[(*V_).rep] = 0;
  }
  
  return D;
}


size_t above_h_articles_k(instance &inst, unsigned h, unsigned k) {
  assert(inst.comps.size() == 1);
  if(inst.comps[0].c < 1) return 0;
  assert(num_vertices(inst.g) == 0);

  table D(d_table_k(inst, h, k));
  mtable T(uint64_t(1) << inst.comps[0].c, k);

  for(unsigned kcur = 0; kcur <= k; ++kcur) T.store(0, kcur, 0);
  comp_subset VC((uint64_t(1) << inst.comps[0].c) - 1);

  SubsetIterator<comp_subset> V_(VC), end;

  for(++V_; V_ != end; ++V_) {
    for(unsigned kcur = 0; kcur <= k; ++kcur) {
      size_t m = 0;

      SubsetIterator<comp_subset> V__(*V_);
      for(; V__ != end; ++V__) {
	comp_subset to_merge = *V_ - *V__;
	unsigned mergesize = to_merge.size();
	unsigned remaink = kcur - (mergesize - 1);

	if(kcur + 1 < mergesize) continue;

	size_t cur = T.get((*V__).rep, remaink) + D[to_merge.rep];
	if(cur > m) m = cur;
      }

      T.store((*V_).rep, kcur, m);
    }
  }
  
  return T.get(VC.rep, k);
}

